@@ -21,8 +21,6 @@
 
 #define FUSE_USE_VERSION 31
 
-#include <config.h>
-
 #include <fuse_lowlevel.h>
 #include <stdio.h>
 #include <unistd.h>
@@ -81,6 +79,8 @@ static void pc_init(void *userdata,
 			printf("\tFUSE_CAP_NO_OPENDIR_SUPPORT\n");
 	if(conn->capable & FUSE_CAP_EXPLICIT_INVAL_DATA)
 			printf("\tFUSE_CAP_EXPLICIT_INVAL_DATA\n");
+	if(conn->capable & FUSE_CAP_EXPIRE_ONLY)
+			printf("\tFUSE_CAP_EXPIRE_ONLY\n");
 	fuse_session_exit(se);
 }
 
