@@ -11,7 +11,7 @@
 
 #define _GNU_SOURCE
 
-#include "config.h"
+#include "fuse_config.h"
 #include "fuse_i.h"
 #include "fuse_lowlevel.h"
 #include <string.h>
