@@ -8,7 +8,7 @@
   See the file COPYING.LIB
 */
 
-#include "config.h"
+#include "fuse_config.h"
 #include "fuse_lowlevel.h"
 #include "fuse_i.h"
 
